/*
   Note: With cache ping-ponging turned on (i.e., with false sharing)
         the performance of this program should decrease
         as you increase the number of cores!
*/
#include <stdio.h>
#include <time.h>
#include <pthread.h>

int main()
{
   #define cores 1        // choose the number of cores, start with 1 core as a baseline
   #define ping_pong 0    // set to 0 for cache ping-ponging, set to 1 for NO ping-ponging
   #define cache_line 16  // number of ints (not bytes) in a single cache line

   int count[cores*cache_line];  // one cache line for each core

   void *thread(void *vargp);    // thread function prototype

   void *parameter;              // pointer to thread parameter

   int start;  // used for timing data
   int end;
   int i;      // loop index

   pthread_t tid[cores];   // stores thread ids

   // Initialize the thread parameters.
   for (i = 0; i < cores*cache_line; i++)
      count[i]=1;

   printf("Starting (pthreads) threads\n");
   start=clock();  // begin timing

   // Create one thread per core.
   // If ping_pong==0, then all the thread parameters are next
   // to each other in the first cache line of the array count[].
   // If ping_pong==1, then the thread parameters are spread out
   // in count[], one parameter per cache line.
   // (Note that these function calls have no error checking.)
   for (i = 0; i < cores; i++)
   {
      parameter = count + i + (ping_pong * i * cache_line);  // pointer arithmetic
      pthread_create(&(tid[i]), NULL, thread, parameter);
   }

   // wait for the threads to finish
   for (i = 0; i < cores; i++)
      pthread_join(tid[i], NULL);

   end=clock();  // end timing
   fprintf(stderr, "\nAll done. time = %d\n", end-start);
   return 0;
}//main()


void *thread(void *threadParam)
{
   int* count = (int*)threadParam;

   while(*count != 0)  // loop until the counter rolls over
   {
      (*count)++;
   }
   return 0;
}//thread function
