/*
   Note: With cache ping-ponging turned on (i.e., with false sharing)
         the performance of this program should decrease
         as you increase the number of cores!
*/
#include <stdio.h>
#include <time.h>
#include <windows.h>

int main()
{
   #define cores 1        // choose the number of cores, start with 1 core as a baseline
   #define ping_pong 0    // set to 0 for cache ping-ponging, set to 1 for NO ping-ponging
   #define cache_line 16  // number of ints (not bytes) in a single cache line

   int count[cores*cache_line];  // one cache line for each core

   DWORD WINAPI thread(LPVOID);  // thread function prototype

   LPVOID lpParameter;           // pointer to thread parameter

   int start;  // used for timing data
   int end;
   int i;      // loop index

   HANDLE hthread[cores];  // stores thread handles

   // Initialize the thread parameters.
   for (i = 0; i < cores*cache_line; i++)
      count[i]=1;

   // Create one thread per core (but do not let them run yet).
   // If ping_pong==0, then all the thread parameters are next
   // to each other in the first cache line of the array count[].
   // If ping_pong==1, then the thread parameters are spread out
   // in count[], one parameter per cache line.
   // (Note that these function calls have no error checking.)
   for (i = 0; i < cores; i++)
   {
      lpParameter = count + i + (ping_pong * i * cache_line);  // pointer arithmetic
      hthread[i] = CreateThread(NULL, 0, thread, lpParameter, CREATE_SUSPENDED, NULL);
   }

   // set the priority of the (suspended) threads
   for (i = 0; i < cores; i++)
      SetThreadPriority(hthread[i], THREAD_PRIORITY_HIGHEST);

   printf("Starting (Win32) threads\n");
   start=clock();  // begin timing

   // now start the threads
   for (i = 0; i < cores; i++)
      ResumeThread(hthread[i]);

   // wait for the threads to finish
   WaitForMultipleObjects(cores, hthread, TRUE, INFINITE);

   end=clock();  // end timing
   fprintf(stderr, "\nAll done. time = %d\n", end-start);
   return 0;
}//main()


DWORD WINAPI thread(LPVOID threadParam)
{
   int* count = (int*)threadParam;

   while(*count != 0)  // loop until the counter rolls over
   {
      (*count)++;
   }
   return 0;
}//thread function
